package com.sun.security.sasl.util;

import java.io.ByteArrayInputStream;
import sun.misc.HexDumpEncoder;
import java.io.ByteArrayOutputStream;
import java.util.StringTokenizer;
import javax.security.sasl.SaslException;
import java.util.logging.Level;
import java.util.Map;
import java.util.logging.Logger;

public abstract class AbstractSaslImpl
{
    protected boolean completed;
    protected boolean privacy;
    protected boolean integrity;
    protected byte[] qop;
    protected byte allQop;
    protected byte[] strength;
    protected int sendMaxBufSize;
    protected int recvMaxBufSize;
    protected int rawSendSize;
    protected String myClassName;
    private static final String SASL_LOGGER_NAME = "javax.security.sasl";
    protected static final String MAX_SEND_BUF = "javax.security.sasl.sendmaxbuffer";
    protected static final Logger logger;
    protected static final byte NO_PROTECTION = 1;
    protected static final byte INTEGRITY_ONLY_PROTECTION = 2;
    protected static final byte PRIVACY_PROTECTION = 4;
    protected static final byte LOW_STRENGTH = 1;
    protected static final byte MEDIUM_STRENGTH = 2;
    protected static final byte HIGH_STRENGTH = 4;
    private static final byte[] DEFAULT_QOP;
    private static final String[] QOP_TOKENS;
    private static final byte[] QOP_MASKS;
    private static final byte[] DEFAULT_STRENGTH;
    private static final String[] STRENGTH_TOKENS;
    private static final byte[] STRENGTH_MASKS;
    
    protected AbstractSaslImpl(Map<String, ?> map, String myClassName) throws SaslException {
        this.completed = false;
        this.privacy = false;
        this.integrity = false;
        this.sendMaxBufSize = 0;
        this.recvMaxBufSize = 65536;
        this.myClassName = myClassName;
        if (map != null) {
            String param1;
            this.qop = parseQop(param1 = (String)map.get("javax.security.sasl.qop"));
            AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL01:Preferred qop property: {0}", param1);
            this.allQop = combineMasks(this.qop);
            if (AbstractSaslImpl.logger.isLoggable(Level.FINE)) {
                AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL02:Preferred qop mask: {0}", new Byte(this.allQop));
                if (this.qop.length > 0) {
                    StringBuffer sb = new StringBuffer();
                    for (int i = 0; i < this.qop.length; ++i) {
                        sb.append(Byte.toString(this.qop[i]));
                        sb.append(' ');
                    }
                    AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL03:Preferred qops : {0}", sb.toString());
                }
            }
            String param2;
            this.strength = parseStrength(param2 = (String)map.get("javax.security.sasl.strength"));
            AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL04:Preferred strength property: {0}", param2);
            if (AbstractSaslImpl.logger.isLoggable(Level.FINE) && this.strength.length > 0) {
                StringBuffer o = new StringBuffer();
                for (int j = 0; j < this.strength.length; ++j) {
                    o.append(Byte.toString(this.strength[j]));
                    o.append(' ');
                }
                AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL05:Cipher strengths: {0}", o.toString());
            }
            String s = (String)map.get("javax.security.sasl.maxbuffer");
            if (s != null) {
                try {
                    AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL06:Max receive buffer size: {0}", s);
                    this.recvMaxBufSize = Integer.parseInt(s);
                }
                catch (NumberFormatException ex) {
                    throw new SaslException("Property must be string representation of integer: javax.security.sasl.maxbuffer");
                }
            }
            String s2 = (String)map.get("javax.security.sasl.sendmaxbuffer");
            if (s2 != null) {
                try {
                    AbstractSaslImpl.logger.logp(Level.FINE, this.myClassName, "constructor", "SASLIMPL07:Max send buffer size: {0}", s2);
                    this.sendMaxBufSize = Integer.parseInt(s2);
                }
                catch (NumberFormatException ex2) {
                    throw new SaslException("Property must be string representation of integer: javax.security.sasl.sendmaxbuffer");
                }
            }
        }
        else {
            this.qop = AbstractSaslImpl.DEFAULT_QOP;
            this.allQop = 1;
            this.strength = AbstractSaslImpl.STRENGTH_MASKS;
        }
    }
    
    public boolean isComplete() {
        return this.completed;
    }
    
    public Object getNegotiatedProperty(String s) {
        if (!this.completed) {
            throw new IllegalStateException("SASL authentication not completed");
        }
        int n = -1;
        switch (s.hashCode()) {
            case -1548608927: {
                if (s.equals("javax.security.sasl.qop")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 1495157683: {
                if (s.equals("javax.security.sasl.maxbuffer")) {
                    n = 1;
                    break;
                }
                break;
            }
            case -2079432448: {
                if (s.equals("javax.security.sasl.rawsendsize")) {
                    n = 2;
                    break;
                }
                break;
            }
            case -1673898581: {
                if (s.equals("javax.security.sasl.sendmaxbuffer")) {
                    n = 3;
                    break;
                }
                break;
            }
        }
        switch (n) {
            case 0: {
                if (this.privacy) {
                    return "auth-conf";
                }
                if (this.integrity) {
                    return "auth-int";
                }
                return "auth";
            }
            case 1: {
                return Integer.toString(this.recvMaxBufSize);
            }
            case 2: {
                return Integer.toString(this.rawSendSize);
            }
            case 3: {
                return Integer.toString(this.sendMaxBufSize);
            }
            default: {
                return null;
            }
        }
    }
    
    protected static final byte combineMasks(byte[] array) {
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            n = (byte)(n | array[i]);
        }
        return n;
    }
    
    protected static final byte findPreferredMask(byte b, byte[] array) {
        int i = 0;
        while (i < array.length) {
            if ((array[i] & b) != 0x0) {
                return array[i];
            }
            ++i;
        }
        return 0;
    }
    
    private static final byte[] parseQop(String s) throws SaslException {
        return parseQop(s, null, false);
    }
    
    protected static final byte[] parseQop(String s, String[] array, boolean b) throws SaslException {
        if (s == null) {
            return AbstractSaslImpl.DEFAULT_QOP;
        }
        return parseProp("javax.security.sasl.qop", s, AbstractSaslImpl.QOP_TOKENS, AbstractSaslImpl.QOP_MASKS, array, b);
    }
    
    private static final byte[] parseStrength(String s) throws SaslException {
        if (s == null) {
            return AbstractSaslImpl.DEFAULT_STRENGTH;
        }
        return parseProp("javax.security.sasl.strength", s, AbstractSaslImpl.STRENGTH_TOKENS, AbstractSaslImpl.STRENGTH_MASKS, null, false);
    }
    
    private static final byte[] parseProp(String str, String s, String[] array, byte[] array2, String[] array3, boolean b) throws SaslException {
        StringTokenizer stringTokenizer = new StringTokenizer(s, ", \t\n");
        byte[] array4 = new byte[array.length];
        int i = 0;
        while (stringTokenizer.hasMoreTokens() && i < array4.length) {
            String nextToken = stringTokenizer.nextToken();
            int n = 0;
            for (int n2 = 0; n == 0 && n2 < array.length; ++n2) {
                if (nextToken.equalsIgnoreCase(array[n2])) {
                    n = 1;
                    array4[i++] = array2[n2];
                    if (array3 != null) {
                        array3[n2] = nextToken;
                    }
                }
            }
            if (n != 0 || b) {
                continue;
            }
            throw new SaslException("Invalid token in " + str + ": " + s);
        }
        for (; i < array4.length; ++i) {
            array4[i] = 0;
        }
        return array4;
    }
    
    protected static final void traceOutput(String s, String s2, String s3, byte[] array) {
        traceOutput(s, s2, s3, array, 0, (array == null) ? 0 : array.length);
    }
    
    protected static final void traceOutput(String s, String s2, String s3, byte[] buf, int offset, int min) {
        try {
            Level level;
            if (!AbstractSaslImpl.logger.isLoggable(Level.FINEST)) {
                min = Math.min(16, min);
                level = Level.FINER;
            }
            else {
                level = Level.FINEST;
            }
            String string;
            if (buf != null) {
                ByteArrayOutputStream o = new ByteArrayOutputStream(min);
                new HexDumpEncoder().encodeBuffer(new ByteArrayInputStream(buf, offset, min), o);
                string = o.toString();
            }
            else {
                string = "NULL";
            }
            AbstractSaslImpl.logger.logp(level, s, s2, "{0} ( {1} ): {2}", new Object[] { s3, new Integer(min), string });
        }
        catch (Exception thrown) {
            AbstractSaslImpl.logger.logp(Level.WARNING, s, s2, "SASLIMPL09:Error generating trace output: {0}", thrown);
        }
    }
    
    protected static final int networkByteOrderToInt(byte[] array, int n, int n2) {
        if (n2 > 4) {
            throw new IllegalArgumentException("Cannot handle more than 4 bytes");
        }
        int n3 = 0;
        for (int i = 0; i < n2; ++i) {
            n3 = (n3 << 8 | (array[n + i] & 0xFF));
        }
        return n3;
    }
    
    protected static final void intToNetworkByteOrder(int n, byte[] array, int n2, int n3) {
        if (n3 > 4) {
            throw new IllegalArgumentException("Cannot handle more than 4 bytes");
        }
        for (int i = n3 - 1; i >= 0; --i) {
            array[n2 + i] = (byte)(n & 0xFF);
            n >>>= 8;
        }
        return;
    }
    
    static {
        logger = Logger.getLogger("javax.security.sasl");
        DEFAULT_QOP = new byte[] { 1 };
        QOP_TOKENS = new String[] { "auth-conf", "auth-int", "auth" };
        QOP_MASKS = new byte[] { 4, 2, 1 };
        DEFAULT_STRENGTH = new byte[] { 4, 2, 1 };
        STRENGTH_TOKENS = new String[] { "low", "medium", "high" };
        STRENGTH_MASKS = new byte[] { 1, 2, 4 };
    }
}
